from typing import Literal, Any, Optional, List, Mapping
from pulumi.output import Output
from theia import Theia
from pulumi import ResourceOptions
from theia.aws.network.vpc import Vpc
from theia.aws.security.sg import SecurityGroup, SecurityGroupSettings
from basetest import testBase

class sgSettings:
    def __init__(self,
                 sg_type: Literal["ingress", "egress"] = "egress",
                 traffic_type: Literal["All","All TCP","All UDP","SSH","SMTP","DNS (UDP)","DNS (TCP)","HTTP","HTTPS","RDP"] = "All",
                 protocol: Literal["-1", "tcp", "udp", "icmp", "icmpv6"] = None,
                 description: Optional[str] = None,
                 add_self: Optional[bool] = False,
                 cidr_blocks: Optional[List[str | Output[str]]] = None):
        
        self.sg_type = sg_type
        self.traffic_type = traffic_type
        self.protocol = protocol
        self.description = description
        self.add_self = add_self
        self.cidr_blocks = cidr_blocks


class dependentBaseTest:
    def __init__(self,
                 project: str,
                 email: str,
                 region: str,
                 backend_url: str,
                 description: Optional[str] = None,
                 sgSettingsList: Optional[List[sgSettings]] = [],
                 tags: Optional[Mapping[str, str]] = None):
        
        self.project = project
        self.email = email
        self.stack = dependentBaseTest.__name__
        self.dependency = testBase.__name__
        self.region = region
        self.backend_url = backend_url
        self.sgSettings = sgSettingsList
        self.description = description

        if tags is None:
            tags = {}

        tags.update({ "Stack" : f"{self.project}-{self.stack}",
                "created_by" : f"{self.email}"})

        self.tags = tags

    def infra(self):

        def program():

            targetVpc = Vpc.import_(stack=f"organization/{self.project}/{self.dependency}" , 
                                    resource_name=f"{self.project}-{self.dependency}-vpc")
            
            settings = self.sgSettings
            securitySettings = []

            for setting in settings:
                securitySetting = SecurityGroupSettings(sg_type=setting.sg_type,
                                                        traffic_type=setting.traffic_type,
                                                        from_port=setting.from_port,
                                                        protocol=setting.protocol,
                                                        to_port=setting.to_port,
                                                        description=setting.description,
                                                        add_self=setting.add_self,
                                                        cidr_blocks=setting.cidr_blocks
                                                        )
                
                securitySettings.append(securitySetting)

            sg = SecurityGroup(resource_name=f"{self.project}-{self.stack}-sg",
                               settings=securitySettings,
                               vpc_id=targetVpc.id,
                               opts=ResourceOptions(depends_on=targetVpc))
            
        return Theia(project=self.project, stack=self.stack, program=program, region=self.region, backend_url=self.backend_url, stack_imports=[f"organization/{self.project}/{self.dependency}"])

    def __call__(self, command: Literal['preview', 'up', 'destroy', 'refresh']) -> Any:
        getattr(self.infra(), command)()